# standard
import time

# PyQGIS
from qgis.core import QgsApplication, QgsExpression, QgsFeatureRequest, QgsVectorLayer
from qgis.gui import QgsFilterLineEdit
from qgis.PyQt.QtCore import QRegExp, Qt
from qgis.PyQt.QtGui import QRegExpValidator
from qgis.PyQt.QtWidgets import QApplication, QComboBox, QCompleter, QWidget

# project
from locator_grand_lyon.toolbelt.log_handler import PlgLogger


class UniqueLayerAttributesComboBox(QComboBox):
    """Widget to select unique layer field"""

    def __init__(self, parent: QWidget) -> None:
        super().__init__(parent)

        self.log = PlgLogger().log

        # Add completer for search
        completer = QCompleter(self.model())
        completer.setWidget(self)
        self.setEditable(True)
        self.setCompleter(completer)

        # Custom line edit for search
        lineedit = QgsFilterLineEdit(None, QgsApplication.nullRepresentation())
        lineedit.setSelectOnFocus(True)
        lineedit.setShowClearButton(True)

        self.setLineEdit(lineedit)

        # No layer and field selected by default
        self.layer = None
        self.field = None

    def setSourceLayer(self, layer: QgsVectorLayer) -> None:
        """Define source layer for feature search

        :param layer: source layer
        :type layer: QgsVectorLayer
        """
        self.layer = layer

    def setDisplayField(self, field: str) -> None:
        """Define display unique field

        :param field: field name
        :type field: str
        """
        self.field = field

    def setFilterExpression(self, expression: str) -> None:
        """Define filter expression for unique field search

        :param expression: expression for search
        :type expression: str
        """
        if self.layer is not None and self.field is not None:
            QApplication.setOverrideCursor(Qt.WaitCursor)
            time_start = time.perf_counter()
            # Get uniques value
            self.layer.setSubsetString(expression)
            fields = self.layer.fields()
            unique_value = self.layer.uniqueValues(fields.indexFromName(self.field))

            time_end = time.perf_counter()
            time_duration = time_end - time_start
            self.log(
                message=f"Filter unique layer field with expression {expression} in {time_duration:.3f} seconds",
                log_level=4,
            )

            # Sort for display
            unique_value = list(unique_value)
            unique_value.sort()

            QApplication.restoreOverrideCursor()

            # Add validator to use only available values
            regex = QRegExp("^(" + "|".join(unique_value) + ")$")
            validator = QRegExpValidator(regex, self)
            self.setValidator(validator)

            # Define combobox values
            self.clear()
            self.addItems(unique_value)
